use clap::{Parser, ValueEnum};

use crate::{
	agents::{Diffuse, Human, MaximizerAgent, MinimizerAgent, Random, SimpleMinimax},
	util::Player,
};

#[derive(Parser, Debug)]
#[command(version, about)]
pub struct Cli {
	pub red: AgentSelector,
	pub blue: AgentSelector,

	#[arg(long, short, default_value = "0")]
	pub repeat: usize,

	#[arg(long, short, default_value = "false")]
	pub silent: bool,
}

#[derive(Debug, Copy, Clone, PartialEq, Eq, PartialOrd, Ord, ValueEnum)]
pub enum AgentSelector {
	Random,
	Diffuse,
	Minimax,
	Human,
}

impl AgentSelector {
	pub fn to_maxi(&self, player: Player) -> Box<dyn MaximizerAgent> {
		match self {
			Self::Random => Box::new(Random::new(player)),
			Self::Minimax => Box::new(SimpleMinimax::new(player)),
			Self::Diffuse => Box::new(Diffuse::new(player)),
			Self::Human => Box::new(Human::new(player)),
		}
	}

	pub fn to_mini(&self, player: Player) -> Box<dyn MinimizerAgent> {
		match self {
			Self::Random => Box::new(Random::new(player)),
			Self::Minimax => Box::new(SimpleMinimax::new(player)),
			Self::Diffuse => Box::new(Diffuse::new(player)),
			Self::Human => Box::new(Human::new(player)),
		}
	}
}

impl ToString for AgentSelector {
	fn to_string(&self) -> String {
		match self {
			Self::Random => "random",
			Self::Diffuse => "diffuse",
			Self::Minimax => "minimax",
			Self::Human => "human",
		}
		.to_string()
	}
}
